/* --------------------------------- heading.c ------------------------------ */

/* This is part of the flight simulator 'fly8'.
 * Author: Eyal Lebedinsky (eyal@ise.canberra.edu.au).
*/

/* paint the Head Up Display: heading.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <stdarg.h>

#include "fly.h"
#include "hud.h"

#undef CP
#undef CW
#undef CS

#define CP	view->viewport
#define CW	view->window
#define CS	view->screen

extern void FAR
show_heading (VIEW *view, OBJECT *p, int sx, int sy, int maxx, int maxy,
	int orgx, int orgy, int ttx, int tty, int tx, int ty, int ss,
	int shifty)
{
	int	hud, hud1, top, big, fine, xfine, fa18, f16, f15, fc, knots;
	int	scale_ref, scale_len, x, y, base, dbase;
	int	s, sl, x0, y0, dm, i, ex, ty1, ty2;
	long	t;
	ANGLE	a;
	LVECT	*R;
	OBJECT	*target;

	hud = EE(p)->hud;
	hud1 = EE(p)->hud1;
	fine = hud & HUD_FINE;
	xfine = hud & HUD_XFINE;
	big = hud & HUD_BIG;
	top = hud1 & HUD_TOP;
	knots = hud1 & HUD_KNOTS;
	i = hud1 & HUD_TYPES;
	fa18 = i == HUD_FA18;
	f16  = i == HUD_F16;
	f15  = i == HUD_F15;
	fc   = i == HUD_CLASSIC;

	if (sx < 100) {
		if (xfine)
			fine = 1;
		xfine = 0;
	}

	if (f15) {
		if (top)
			scale_len = 20;
		else
			scale_len = 16;
		scale_ref = 20;
		sl = fmul (sx, F15HEADS);
	} else if (f16) {
		scale_ref = 16;
		scale_len = 16;
		sl = fmul (sx, F16HEADS);
	} else if (fa18) {
		scale_ref = scale_len = 16;
		sl = fmul (sx, F18HEADS);
	} else {
#if 0
/*
 * This logic will show true heading (the hud will match the scenery). The
 * problem is that the hud is too narrow for comfortable reading.
*/
		x = muldiv (CP->maxx, sl, maxx);
		a = ATAN (x, CP->z);
		scale_ref = ANG2DEG (a);	/* HUD width in degrees */
		if (scale_ref < 1)
			scale_ref = 1;
		scale_len = (scale_ref * 25) / 30;
#else
		scale_ref = 25;
		scale_len = 16;
#endif
		sl = sx;
	}

	ty1 = -tty;
	ty2 = 2*ty1;
	if (f15) {
		if (top)
			base = orgy - fmul (sy, F15HEADH);
		else
			base = orgy - fmul (sy, F15HEADL);
		dbase = base+ty2;
	} else if (f16) {
		if (top)
			base = orgy - fmul (sy, F16HEAD);
		else
			base = orgy + muldiv (sy, 8, 16) + ss;
		dbase = base+ty2;
	} else if (fa18) {
		base = orgy - fmul (sy, (EE(p)->equip & EQ_GEAR)
					? F18HEADG : F18HEAD);
		dbase = base+ty2;
	} else if (top) {
		base = orgy - sy + shifty;
		dbase = base+ty2;
		if (big)
			dbase += ss;
	} else {
		base = orgy + sy + shifty;
		ty1 = -ty1;
		ty2 = -ty2;
		dbase = base+ty2;
		if (!big)
			dbase += ss;
	}
/*
 * show aoa on left.
*/
	if (fc) {
		if (hud1 & HUD_TOP)
			y = base + ss;
		else
			y = base;

		x = orgx - sx;
		stroke_char (x, y, 'A', ss, st.hfg);
		stroke_frac (x+ss, y, ANG2DEG00(EE(p)->misc[7])/10,
			0, 1, ss, st.hfg);
	}

	if (!(EE(p)->hud2 & HUD_HEADING))
		return;
/*
 * Show heading scale.
*/
	a = -p->a[Z];
	if (a >= 0) {
		y0 = (int)(a % (D90/9));
		ex = 10*(int)(a / (D90/9));
	} else {
		a = -a;
		y0 = D90/9 - (int)(a % (D90/9));
		ex = -10*(int)(a / (D90/9)) - 10;
	}

	dm = num_size (9L, ss);
	if (hud&HUD_FULLHEADING)
		dm = dm * 3 / 2;

	s = muldiv (y0, 90, D90);		/* degrees */
	x0 = -y0 + muldiv (s, D90, 90);		/* fraction */
	x0 = orgx + muldiv (x0, sl, D90/90*scale_ref);
	if (f15 || (fa18 && top)) {
		x = muldiv (scale_len, sl, scale_ref);
		add_line (orgx - x, base, T_MOVE);
		add_line (orgx + x, base, st.hfg);
	}
	for (i = 1-scale_len, s += i; i <= scale_len; ++i, ++s) {
		x = x0 + muldiv (i, sl, scale_ref);
		if (0 == s%10) {
			add_line (x, base, T_MOVE);
			add_line (x, base+ty2, st.hfg);
			t = (ex + s + 360) % 360;
			if (hud&HUD_FULLHEADING)
				stroke_frac (x-dm, dbase, t, 3, 0, ss, st.hfg);
			else if (!fc)
				stroke_frac (x-dm, dbase, t/10, 2, 0, ss,
					st.hfg);
			else if (xfine || (i >= -11 && i <= 11)) {
				dm = num_size (t, ss);
				stroke_num (x-dm/2, dbase, t, ss, st.hfg);
			}
		} else if (fine) {
			if (0 == s%2) {
				add_line (x, base, T_MOVE);
				add_line (x, base+ty1, st.hfg);
			}
		} else if (0 == s%5) {
			add_line (x, base, T_MOVE);
			add_line (x, base+(xfine?ty2:ty1), st.hfg);
		} else if (xfine) {
			add_line (x, base, T_MOVE);
			add_line (x, base+ty1, st.hfg);
		}
	}

	if (fa18||f15) {
		x = tx + tx/2;
		add_line (orgx-x, base+2*ty, T_MOVE);	/* inverted V */
		add_line (orgx,   base,      st.hfg);
		add_line (orgx+x, base+2*ty, st.hfg);
	} else {
		y = base;
		if (big) {
			y += ty2;
			if (top)
				y += ss;
			else
				y -= ss;
			y0 = ty1;
		} else
			y0 = -ty1;

		add_line (orgx, y,      T_MOVE);	/* reading mark */
		add_line (orgx, y+3*y0, st.hfg);
	}
/*
 * show runway/target heading.
*/
	if (((target = EE(p)->target)) && target->id == EE(p)->tid)
		R = &target->R;
	else if (EE(p)->hud2 & HUD_ILS)
		R = &ils[EE(p)->ils-1].R;
	else
		R = 0;

	if (R) {
		long		hx, hy;
		ANGLE		ref;

		hx = ((*R)[X] - p->R[X])/VONE;
		hy = ((*R)[Y] - p->R[Y])/VONE;
		i = ihypot2d ((int)(hx/100), (int)(hy/100));
		if (knots)
			EE(p)->misc[16] = fmul (i, FONE/18*10);
		else
			EE(p)->misc[16] = i;
		t = labs(hx) + labs(hy);
		while (t > 0x7fffL) {
			hx >>= 1;
			hy >>= 1;
			t  >>= 1;
		}
		a = ATAN ((int)hx, (int)hy);
		EE(p)->misc[15] = a;
		a += p->a[Z];
		ref = DEG2ANG (scale_len);
		if (a > ref)
			a = ref;
		else if (a < -ref)
			a = -ref;

		ref = DEG2ANG (scale_ref);
		x = orgx + muldiv ((int)a, sl, (int)ref);
		y = fa18 ? -ty2 : ty1;
		add_line (x-tx, base+y, T_MOVE);
		add_line (x,    base,   st.hfgi);
		add_line (x+tx, base+y, st.hfgi);
	}
}
