/* --------------------------------- ochute.c ------------------------------- */

/* This is part of the flight simulator 'fly8'.
 * Author: Eyal Lebedinsky (eyal@ise.canberra.edu.au).
*/

/* object: parachute.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "fly.h"

static SHAPE shape_chute = {
	0,
	0,
	SH_G|SH_LOCALSIM,
	200*1000L,	/* weight */
	0		/* drag */
};

static int FAR
init_chute (BODY *b)
{
	if (shape_read (&shape_chute, "chute.vxx"))
		return (1);
	return (0);
}

static void FAR
term_chute (BODY *b)
{
	shape_free (shape_chute.v);
}

static int FAR
create_chute (OBJECT *p)
{
	if (!st.owner)
		return (1);

	p->color = st.green;
	p->time = FOREVER;		/* let them land */
	p->flags |= F_VISIBLE;
	p->speed = st.owner->speed;
	p->owner   = st.owner;		/* point at plane */
	p->ownerid = st.owner->id;

	st.owner->owner = p;		/* it owns the plane */
	st.owner->ownerid = p->id;
	if (st.owner->flags & F_EXPORTED)
		p->flags |= F_EXPORTED;

	Vcopy (p->V, st.owner->V);
	LVcopy (p->R, st.owner->R);
	p->da[Z] = VD90/18;
	Mident (p->T);

	return (0);
}

extern void FAR
dynamics_chute (OBJECT *p, int interval)
{
	int	force;

	if (p->R[Z] <= 0) {
		p->R[Z] = 0;
		p->V[Z] = 0;
		if (p->ownerid != p->owner->id &&	/* owner gone */
		    !(p->flags & F_IMPORTED)) {		/* native */
			if (p->gpflags & GPF_PILOT) {	/* not a drone */
				CC->gpflags |= GPF_PILOT;
				if (!(st.flags & SF_BLANKER)) {
					CC->flags &= ~F_STEALTH;
					CC->flags |= F_MOD;
				}
			}
			p->flags |= F_DEL|F_MOD;
		}
		return;
	}
	p->V[X] -= TADJ(p->V[X]/2);
	p->V[Y] -= TADJ(p->V[Y]/2);

	p->da[Z] -= TADJ(p->da[Z]/2);

	force = -p->V[Z]*2 - GACC;	/* stabilize at 5 m/sec */
					/* too fast but looks OK */
	p->V[Z] += TADJ (force);
	p->speed = ihypot3d (p->V);

	if (p->da[X] || p->da[Y] || p->da[Z]) {
		p->a[X] += TADJ(p->da[X])*VONE;
		p->a[Y] += TADJ(p->da[Y])*VONE;
		p->a[Z] += TADJ(p->da[Z])*VONE;
		Mobj (p);
	}
}

BODY BoChute = {
	0,
	0,
	"CHUTE",
	&shape_chute,
	init_chute,
	term_chute,
	create_chute,
	body_delete,
	dynamics_chute,
	body_hit
};
