/* --------------------------------- sky.c ---------------------------------- */

/* This is part of the flight simulator 'fly8'.
 * Author: Eyal Lebedinsky (eyal@ise.canberra.edu.au).
*/

/* Render the sky algorithmically.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "fly.h"

#define NSIZE	(st.SkyLines*2+1)

static ANGLE	*skylines = 0;

extern int FAR
sky_init (void)
{
	int	i;
	ANGLE	a, v;

	skylines = (ANGLE *) memory_alloc (NSIZE * sizeof (*skylines));
	if (!skylines) {
		MsgEPrintf (-50, "No Sky");
		st.flags &= ~SF_SKY;
		return (1);
	}
	skylines[0] = 0;
	skylines[st.SkyLines] = D90;
	skylines[2*st.SkyLines] = D180;

	for (i = 1; i < st.SkyLines; ++i) {
		a = muldiv (i, D90, st.SkyLines);
		v = fmul (D90, SIN (a));
		skylines[st.SkyLines-i] = D90 - v;
		skylines[st.SkyLines+i] = D90 + v;
	}
	return (0);
}

extern void FAR
sky_term (void)
{
	memory_free (skylines, NSIZE * sizeof (*skylines));
	skylines = 0;
}

#undef CP
#undef CW
#undef CS

#define CP	view->viewport
#define CW	view->window
#define CS	view->screen

extern void FAR
show_sky (VIEW *view, OBJECT *p, int color, int shift)
{
	int	orgx, orgy, maxx, maxy, sizex, sizey;
	int	sroll, croll, srollh, crollh, spitch, cpitch;
	int	h, xh, yh;
	int	i, k, n, x[2], y[2], slope;
	ANGLE	a, z;

	maxx = CP->maxx;
	maxy = CP->maxy;

	i = ihypot2d (maxx, maxy);
	a = ATAN (i, CP->z);

	orgx = fmul (CW->orgx, CS->sizex) + CS->minx;	/* screen center */
	orgy = fmul (CW->orgy, CS->sizey) + CS->miny;

	sizex = fmul (CW->maxx - iabs(shift), CS->sizex)-1; /* screen size */
	sizey = fmul (CW->maxy, CS->sizey)-1;

	sroll  = SIN (-p->a[Y]);
	croll  = COS (-p->a[Y]);

	i = p->a[X] - a;
	for (n = 0; n < NSIZE; ++n)
		if (skylines[n] >= i)
			break;
/*	a = a/2*3;*/

	srollh = muldiv (sroll, sizex, maxx);
	crollh = muldiv (croll, sizey, maxy);

	slope = iabs(fmul (croll, sizey)) > iabs(fmul (sroll, sizex));

	sroll  = muldiv (sroll, sizey, maxy);
	croll  = muldiv (croll, sizex, maxx);

    for (; n < NSIZE; ++n) {
	z = skylines[n]			/* angle being shown */
	    -p->a[X];			/* relative pitch of line */
	if (iabs (z) > (Uint)a)		/* trivial reject */
		break;

	spitch = SIN (z);
	cpitch = COS (z);

	h = muldiv (CP->z, spitch, cpitch);

	xh = fmul (h, srollh);
	yh = fmul (h, crollh);

	if (!croll) {
		if (xh > sizex || xh < -sizex)
			continue;
		x[0] = xh;
		y[0] = sizey;
		x[1] = xh;
		y[1] = -sizey;
	} else if (!sroll) {
		if (yh > sizey || yh < -sizey)
			continue;
		x[0] = sizex;
		y[0] = yh;
		x[1] = -sizex;
		y[1] = yh;
	} else {
		for (i = 0, k = 0; k < 2; ++k) {
			if (slope^k) {
				if (!k || xh >= 0) {
					x[i] = sizex;
					y[i] = yh-muldiv (x[i]-xh, sroll,
								croll);
					if (y[i] <= sizey && y[i] >= -sizey)
						if (++i >= 2)
							break;
				}
				if (!k || xh < 0) {
					x[i] = -sizex;
					y[i] = yh-muldiv (x[i]-xh, sroll,
								croll);
					if (y[i] <= sizey && y[i] >= -sizey)
						if (++i >= 2)
							break;
				}
				if (0 == i)
					break;
			} else {
				if (!k || yh >= 0) {
					y[i] = sizey;
					x[i] = xh-muldiv (y[i]-yh, croll,
								sroll);
					if (x[i] <= sizex && x[i] >= -sizex)
						if (++i >= 2)
								break;
				}
				if (!k || yh < 0) {
					y[i] = -sizey;
					x[i] = xh-muldiv (y[i]-yh, croll,
							sroll);
					if (x[i] <= sizex && x[i] >= -sizex)
						if (++i >= 2)
							break;
				}
				if (0 == i)
					break;
			}
		}
		if (i < 2)
			continue;
	}
	add_line (orgx+x[0], orgy-y[0], T_MOVE);
	add_line (orgx+x[1], orgy-y[1], color);
    }
}
